"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("../Server");
const services = require("../services");
const chai_1 = require("chai");
const tmp = require("tmp");
const path = require("path");
const superagent = require("superagent");
const uuid = require("uuid");
describe('User Metadata', function () {
    let tmpDir;
    let server;
    const HOST = '127.0.0.1';
    const PORT = 6789;
    let usernamePasswordAuthService;
    let adminUser;
    before(() => __awaiter(this, void 0, void 0, function* () {
        tmpDir = tmp.dirSync();
        Realm.clearTestState();
        server = new Server_1.Server({
            port: PORT,
            logLevel: 'off',
            dataPath: tmpDir.name
        });
        usernamePasswordAuthService = new services.UsernamePasswordAuthService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        });
        server.addService(usernamePasswordAuthService);
        server.addService(new services.RealmManagementService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        }));
        server.addService(new services.SyncProxyService());
        server.addService(new services.SyncService({
            dataPath: path.join(server.config.dataPath, "sync"),
            publicKeyPath: path.join(server.config.dataPath, "keys/auth.pub"),
        }));
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        server.shutdown();
    }));
    it('should be able to get a user programmatically', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'original';
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
            const fetchedUser = usernamePasswordAuthService.getUserById(USERNAME);
            chai_1.expect(fetchedUser).not.to.be.undefined;
            chai_1.expect(fetchedUser.userId).to.be.eq(USERNAME);
        });
    });
    it('should be able get a user via REST API', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'original';
            const EMAIL = `${USERNAME}@realm.io`;
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
            const response = yield superagent.get(`http://${HOST}:${PORT}/auth/users/${USERNAME}`);
            chai_1.expect(response.body).not.to.be.undefined;
            chai_1.expect(response.body["userId"]).to.eq(USERNAME);
        });
    });
    it('should be able to update a user programmatically', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const EMAIL = `${USERNAME}@realm.io`;
            const update = usernamePasswordAuthService
                .createOrUpdateUser(USERNAME, true, { 'email': EMAIL });
            chai_1.expect(update).not.to.be.undefined;
            chai_1.expect(update.user).not.to.be.undefined;
            chai_1.expect(update.created).to.be.true;
            chai_1.expect(update.user.metadata[0]['key']).to.eq('email');
            chai_1.expect(update.user.metadata[0]['value']).to.eq(EMAIL);
        });
    });
    it('should be able to create or update a user via REST API', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const EMAIL = `${USERNAME}@realm.io`;
            const response = yield superagent.put(`http://${HOST}:${PORT}/auth/users`)
                .send({
                userId: USERNAME,
                isAdmin: true,
                metadata: {
                    'email': EMAIL
                }
            });
            chai_1.expect(response.body).not.to.be.undefined;
            chai_1.expect(response.body["user"]).not.to.be.undefined;
            chai_1.expect(response.body["created"]).to.be.true;
            chai_1.expect(response.body["user"]["userId"]).to.eq(USERNAME);
            chai_1.expect(response.body["user"]["isAdmin"]).to.eq(true);
        });
    });
    it("should be able to change a users password programmatically", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'originalpassword';
            const NEWPASSWORD = 'newpassword';
            const BADPASSWORD = 'badpassword';
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
            yield usernamePasswordAuthService.changePassword(USERNAME, ORIGINALPASSWORD, NEWPASSWORD);
            const { userId } = yield usernamePasswordAuthService.attemptToLogin(USERNAME, NEWPASSWORD);
        });
    });
    it("should be able to change a users password via REST API", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'originalpassword';
            const NEWPASSWORD = 'newpassword';
            const BADPASSWORD = 'badpassword';
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
            const response = yield superagent.put(`http://${HOST}:${PORT}/auth/users/${USERNAME}/password`)
                .send({
                currentPassword: ORIGINALPASSWORD,
                newPassword: NEWPASSWORD
            });
            chai_1.expect(response.status).to.eq(200);
        });
    });
    it("should respond with a 611 error code with a bad change password request programmatically", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'originalpassword';
            const NEWPASSWORD = 'newpassword';
            const BADPASSWORD = 'badpassword';
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD, false);
            yield usernamePasswordAuthService.changePassword(USERNAME, ORIGINALPASSWORD, NEWPASSWORD);
            try {
                yield usernamePasswordAuthService.attemptToLogin(USERNAME, BADPASSWORD);
            }
            catch (err) {
                chai_1.expect(err).to.not.be.undefined;
                chai_1.expect(err["status"]).to.be.eq(400);
                chai_1.expect(err["code"]).to.be.eq(611);
            }
        });
    });
    it("should respond with a 611 error code with a bad change password request via REST API", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const USERNAME = uuid.v4();
            const ORIGINALPASSWORD = 'originalpassword';
            const NEWPASSWORD = 'newpassword';
            const BADPASSWORD = 'badpassword';
            yield usernamePasswordAuthService.attemptToRegister(USERNAME, ORIGINALPASSWORD);
            try {
                const response = yield superagent.put(`http://${HOST}:${PORT}/auth/users/${USERNAME}/password`)
                    .send({
                    currentPassword: BADPASSWORD,
                    newPassword: NEWPASSWORD
                });
            }
            catch (err) {
                chai_1.expect(err).to.not.be.undefined;
                chai_1.expect(err["status"]).to.be.eq(400);
                chai_1.expect(err["response"]["body"]["code"]).to.be.eq(611);
            }
        });
    });
});
//# sourceMappingURL=user-metadata-integration.spec.js.map